Package: moocore
Type: Package
Title: Core Mathematical Functions for Multi-Objective Optimization
Version: 0.3.1
Authors@R: c(person("Manuel", "López-Ibáñez", role = c("aut", "cre"),
                    email = "manuel.lopez-ibanez@manchester.ac.uk",
                    comment = c(ORCID = "0000-0001-9974-1295")),
             person("Carlos", "Fonseca", role = "ctb"),
             person("Luís", "Paquete", role = "ctb"),
             person(c("Andreia", "P."), "Guerreiro", role = "ctb"),
             person("Mickaël", "Binois", role = "ctb"),
             person(c("Michael", "H."), "Buselli", role = "cph", comment = "AVL-tree library"),
             person("Wessel", "Dankers", role = "cph", comment = "AVL-tree library"),
             person("NumPy Developers", role = "cph", comment = "RNG and ziggurat constants"),
             person("Jean-Sebastien", "Roy", role = "cph", comment = "mt19937 library"),
             person("Makoto", "Matsumoto", role = "cph", comment = "mt19937 library"),
             person("Takuji", "Nishimura", role = "cph", comment = "mt19937 library"))
Description: Fast implementations of mathematical operations and performance metrics for multi-objective optimization, including filtering and ranking of dominated vectors according to Pareto optimality, hypervolume metric, C.M. Fonseca, L. Paquete, M. López-Ibáñez (2006) <doi:10.1109/CEC.2006.1688440>, epsilon indicator, inverted generational distance, computation of the empirical attainment function, V.G. da Fonseca, C.M. Fonseca, A.O. Hall (2001) <doi:10.1007/3-540-44719-9_15>, and  Vorob'ev threshold, expectation and deviation, M. Binois, D. Ginsbourger, O. Roustant (2015) <doi:10.1016/j.ejor.2014.07.032>, among others.
Depends: R (>= 4.1)
Imports: matrixStats, Rdpack (>= 2.4)
Suggests: doctest (>= 0.4.0), spelling, testthat (>= 3.2.0), withr
License: LGPL (>= 2.1)
Copyright: file COPYRIGHTS
URL: https://multi-objective.github.io/moocore/r/,
        https://github.com/multi-objective/moocore
BugReports: https://github.com/multi-objective/moocore/issues
LazyLoad: true
LazyData: true
Encoding: UTF-8
UseLTO: true
RoxygenNote: 7.3.3
SystemRequirements: GNU make
RdMacros: Rdpack
Config/testthat/edition: 3
Language: en-GB
Config/Needs/website: Hmisc, data.table, dplyr, ggplot2, htmltools,
        kableExtra, knitr, plotly, rmarkdown, scales, tidyr
NeedsCompilation: yes
Packaged: 2026-05-04 07:15:31 UTC; manu
Author: Manuel López-Ibáñez [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-9974-1295>),
  Carlos Fonseca [ctb],
  Luís Paquete [ctb],
  Andreia P. Guerreiro [ctb],
  Mickaël Binois [ctb],
  Michael H. Buselli [cph] (AVL-tree library),
  Wessel Dankers [cph] (AVL-tree library),
  NumPy Developers [cph] (RNG and ziggurat constants),
  Jean-Sebastien Roy [cph] (mt19937 library),
  Makoto Matsumoto [cph] (mt19937 library),
  Takuji Nishimura [cph] (mt19937 library)
Maintainer: Manuel López-Ibáñez <manuel.lopez-ibanez@manchester.ac.uk>
Repository: CRAN
Date/Publication: 2026-05-04 13:10:03 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-04 23:56:49 UTC; unix
