# Generated by roxygen2: do not edit by hand

export(any_dominated)
export(as_double_matrix)
export(attsurf2df)
export(avg_hausdorff_dist)
export(choose_eafdiff)
export(compute_eaf_call)
export(compute_eafdiff_call)
export(eaf)
export(eaf_as_list)
export(eafdiff)
export(epsilon_additive)
export(epsilon_mult)
export(filter_dominated)
export(generate_ndset)
export(hv_approx)
export(hv_contributions)
export(hypervolume)
export(igd)
export(igd_plus)
export(is_nondominated)
export(largest_eafdiff)
export(normalise)
export(pareto_rank)
export(r2_exact)
export(rbind_datasets)
export(read_datasets)
export(total_whv_rect)
export(transform_maximise)
export(vorob_dev)
export(vorob_t)
export(whv_hype)
export(whv_rect)
export(write_datasets)
importFrom(Rdpack,reprompt)
importFrom(matrixStats,colRanges)
importFrom(stats,rexp)
importFrom(stats,rnorm)
importFrom(utils,modifyList)
importFrom(utils,tail)
importFrom(utils,write.table)
useDynLib(moocore, .registration = TRUE)
