# Generated by doctest: do not edit by hand
# Please edit file in R/vorob.R

test_that("Doctest: vorob_t", {
  # Created from @doctest for `vorob_t`
  # Source file: R/vorob.R
  data(CPFs)
  res <- vorob_t(CPFs, reference = c(2, 200))
  expect_equal(res$threshold, 44.140625)
  expect_equal(res$avg_hyp, 8943.3332)
  vd <- vorob_dev(CPFs, ve = res$ve, reference = c(2, 200))
  expect_equal(vd, 3017.1299)
})

