# Generated by doctest: do not edit by hand
# Please edit file in R/nondominated.R

test_that("Doctest: is_nondominated", {
  # Created from @doctest for `is_nondominated`
  # Source file: R/nondominated.R
  S = matrix(c(1, 1, 0, 1, 1, 0, 1, 0), ncol = 2, byrow = TRUE)
  expect_equal(is_nondominated(S), c(FALSE, TRUE, TRUE, FALSE))
  expect_equal(is_nondominated(S, maximise = TRUE), c(TRUE, FALSE, FALSE, FALSE))
  expect_equal(filter_dominated(S), matrix(c(0, 1, 1, 0), ncol = 2, byrow = TRUE))
  expect_equal(filter_dominated(S, keep_weakly = TRUE), matrix(c(0, 1, 1, 0, 1, 0),
  ncol = 2, byrow = TRUE))
  expect_equal(any_dominated(S), TRUE)
  expect_equal(any_dominated(S, keep_weakly = TRUE), TRUE)
  expect_equal(any_dominated(filter_dominated(S)), FALSE)
})

